@@ -69,10 +69,7 @@
 #include <librsvg/rsvg.h>
 #endif
 
-#define EOM_IMAGE_GET_PRIVATE(object) \
-	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOM_TYPE_IMAGE, EomImagePrivate))
-
-G_DEFINE_TYPE (EomImage, eom_image, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (EomImage, eom_image, G_TYPE_OBJECT)
 
 enum {
 	SIGNAL_CHANGED,
@@ -277,14 +274,12 @@ eom_image_class_init (EomImageClass *klass)
 						     NULL, NULL,
 						     g_cclosure_marshal_VOID__VOID,
 						     G_TYPE_NONE, 0);
-
-	g_type_class_add_private (object_class, sizeof (EomImagePrivate));
 }
 
 static void
 eom_image_init (EomImage *img)
 {
-	img->priv = EOM_IMAGE_GET_PRIVATE (img);
+	img->priv = eom_image_get_instance_private (img);
 
 	img->priv->file = NULL;
 	img->priv->image = NULL;
@@ -299,7 +294,6 @@ eom_image_init (EomImage *img)
 	g_mutex_init (&img->priv->status_mutex);
 	img->priv->status = EOM_IMAGE_STATUS_UNKNOWN;
 	img->priv->metadata_status = EOM_IMAGE_METADATA_NOT_READ;
-	img->priv->is_monitored = FALSE;
 	img->priv->undo_stack = NULL;
 	img->priv->trans = NULL;
 	img->priv->trans_autorotate = NULL;
@@ -682,17 +676,19 @@ eom_image_set_icc_data (EomImage *img, EomMetadataReader *md_reader)
 }
 #endif
 
-#ifdef HAVE_EXIF
 static void
 eom_image_set_orientation (EomImage *img)
 {
 	EomImagePrivate *priv;
+#ifdef HAVE_EXIF
 	ExifData* exif;
+#endif
 
 	g_return_if_fail (EOM_IS_IMAGE (img));
 
 	priv = img->priv;
 
+#ifdef HAVE_EXIF
 	exif = (ExifData*) eom_image_get_exif_info (img);
 
 	if (exif != NULL) {
@@ -704,10 +700,27 @@ eom_image_set_orientation (EomImage *img)
 		if (entry && entry->data != NULL) {
 			priv->orientation = exif_get_short (entry->data, o);
 		}
-	}
+		exif_data_unref (exif);
+	} else
+#endif
+	{
+		GdkPixbuf *pbuf;
 
-	/* exif_data_unref handles NULL values like g_free */
-	exif_data_unref (exif);
+		pbuf = eom_image_get_pixbuf (img);
+
+		if (pbuf) {
+			const gchar *o_str;
+
+			o_str = gdk_pixbuf_get_option (pbuf, "orientation");
+			if (o_str) {
+				short t = (short) g_ascii_strtoll (o_str,
+								   NULL, 10);
+				if (t >= 0 && t < 9)
+					priv->orientation = t;
+			}
+			g_object_unref (pbuf);
+		}
+	}
 
 	if (priv->orientation > 4 &&
 	    priv->orientation < 9) {
@@ -756,7 +769,6 @@ eom_image_autorotate (EomImage *img)
 	/* Schedule auto orientation */
 	img->priv->autorotate = TRUE;
 }
-#endif
 
 #ifdef HAVE_EXEMPI
 static void
@@ -1009,8 +1021,6 @@ eom_image_real_load (EomImage *img,
 					break;
 				}
 
-				eom_image_emit_size_prepared (img);
-
 				priv->metadata_status = EOM_IMAGE_METADATA_NOT_AVAILABLE;
 			}
 
@@ -1126,6 +1136,14 @@ eom_image_real_load (EomImage *img,
 			}
 
 			priv->file_is_changed = FALSE;
+
+			/* Set orientation again for safety, eg. if we don't
+			 * have Exif data or HAVE_EXIF is undefined. */
+			if (priv->autorotate) {
+				eom_image_set_orientation (img);
+				eom_image_emit_size_prepared (img);
+			}
+
 		} else {
 			/* Some loaders don't report errors correctly.
 			 * Error will be set below. */
@@ -1222,16 +1240,16 @@ eom_image_load (EomImage *img, EomImageData data2read, EomJob *job, GError **err
 
 	success = eom_image_real_load (img, data2read, job, error);
 
-#ifdef HAVE_EXIF
 	/* Check that the metadata was loaded at least once before
 	 * trying to autorotate. Also only an imatge load job should try to
 	 * autorotate and image */
-	if (priv->autorotate && 
-	    priv->metadata_status == EOM_IMAGE_METADATA_READY &&
-	      data2read & EOM_IMAGE_DATA_IMAGE) {
-		eom_image_real_autorotate (img);
-	}
+	if (priv->autorotate &&
+#ifdef HAVE_EXIF
+	    priv->metadata_status != EOM_IMAGE_METADATA_NOT_READ &&
 #endif
+	    data2read & EOM_IMAGE_DATA_IMAGE) {
+	                          eom_image_real_autorotate (img);
+	}
 
 	if (success && eom_image_needs_transformation (img)) {
 		success = eom_image_apply_transformations (img, error);
@@ -2011,14 +2029,14 @@ eom_image_get_exif_info (EomImage *img)
 gpointer
 eom_image_get_xmp_info (EomImage *img)
 {
-	EomImagePrivate *priv;
  	gpointer data = NULL;
 
  	g_return_val_if_fail (EOM_IS_IMAGE (img), NULL);
 
+#ifdef HAVE_EXEMPI
+	EomImagePrivate *priv;
  	priv = img->priv;
 
-#ifdef HAVE_EXEMPI
 	g_mutex_lock (&priv->status_mutex);
  	data = (gpointer) xmp_copy (priv->xmp);
 	g_mutex_unlock (&priv->status_mutex);
