@@ -85,16 +85,14 @@ struct _EomMetadataReaderPngPrivate {
 	gboolean   hasIHDR;
 };
 
-#define EOM_METADATA_READER_PNG_GET_PRIVATE(object) \
-	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOM_TYPE_METADATA_READER_PNG, EomMetadataReaderPngPrivate))
-
 static void
 eom_metadata_reader_png_init_emr_iface (gpointer g_iface, gpointer iface_data);
 
 G_DEFINE_TYPE_WITH_CODE (EomMetadataReaderPng, eom_metadata_reader_png,
 			 G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (EOM_TYPE_METADATA_READER,
-			 		eom_metadata_reader_png_init_emr_iface))
+			                        eom_metadata_reader_png_init_emr_iface) \
+			                        G_ADD_PRIVATE(EomMetadataReaderPng))
 
 static void
 eom_metadata_reader_png_dispose (GObject *object)
@@ -121,11 +119,11 @@ eom_metadata_reader_png_dispose (GObject *object)
 }
 
 static void
-eom_metadata_reader_png_init (EomMetadataReaderPng *obj)
+eom_metadata_reader_png_init (EomMetadataReaderPng *emr)
 {
 	EomMetadataReaderPngPrivate *priv;
 
-	priv = obj->priv =  EOM_METADATA_READER_PNG_GET_PRIVATE (obj);
+	priv = emr->priv =  eom_metadata_reader_png_get_instance_private (emr);
 	priv->icc_chunk = NULL;
 	priv->icc_len = 0;
 	priv->xmp_chunk = NULL;
@@ -148,8 +146,6 @@ eom_metadata_reader_png_class_init (EomMetadataReaderPngClass *klass)
 	GObjectClass *object_class = (GObjectClass*) klass;
 
 	object_class->dispose = eom_metadata_reader_png_dispose;
-
-	g_type_class_add_private (klass, sizeof (EomMetadataReaderPngPrivate));
 }
 
 static gboolean
